<?php

namespace Nest\Cli;

/**
 * CLI out functions.
 */
class Out
{
	/**
	 * Reset attributes special character.
	 */
	public const string RESET_ATTRIBUTES = "\e[0m";
	/**
	 * Bold attribute special character.
	 */
	public const string BOLD_ATTRIBUTE = "\e[1m";
	/**
	 * Italic attribute special character.
	 */
	public const string ITALIC_ATTRIBUTE = "\e[3m";
	/**
	 * Dim attribute special character.
	 */
	public const string DIM_ATTRIBUTE = "\e[2m";

	/**
	 * Print a message.
	 * @param string $message Message to print.
	 * @param string $symbol Prefix symbol.
	 * @param Color $color Text color.
	 * @param bool $bold Set text to bold.
	 * @param bool $dim Set text to dim.
	 * @return void
	 */
	public static function print(string $message, string $symbol = "❯", Color $color = Color::Default, bool $bold = false, bool $dim = false): void
	{
		echo
			// Set color and attributes.
			$color->value.($bold ? self::BOLD_ATTRIBUTE : "").($dim ? self::DIM_ATTRIBUTE : "").
			// Print message with its symbol.
			"$symbol $message".
			// Reset colors and attributes.
			Color::Default->value.self::RESET_ATTRIBUTES."\n";
	}

	/**
	 * Print an informational message.
	 * @param string $message Message to print.
	 * @param bool $bold Set text to bold.
	 * @param bool $dim Set text to dim.
	 * @return void
	 */
	public static function info(string $message, bool $bold = false, bool $dim = false): void
	{
		self::print($message, "ℹ", Color::Cyan, $bold, $dim);
	}

	/**
	 * Print a success message.
	 * @param string $message Message to print.
	 * @param bool $bold Set text to bold.
	 * @param bool $dim Set text to dim.
	 * @return void
	 */
	public static function success(string $message, bool $bold = false, bool $dim = false): void
	{
		self::print($message, "✔", Color::Green, $bold, $dim);
	}

	/**
	 * Print a warning message.
	 * @param string $message Message to print.
	 * @param bool $bold Set text to bold.
	 * @param bool $dim Set text to dim.
	 * @return void
	 */
	public static function warning(string $message, bool $bold = false, bool $dim = false): void
	{
		self::print($message, "!", Color::Yellow, $bold, $dim);
	}

	/**
	 * Print an error message.
	 * @param string $message Message to print.
	 * @param bool $bold Set text to bold.
	 * @param bool $dim Set text to dim.
	 * @return void
	 */
	public static function error(string $message, bool $bold = false, bool $dim = false): void
	{
		self::print($message, "‼", Color::Red, $bold, $dim);
	}
}
